// WClient - Willem's Java WebClient

import java.io.*;
import java.net.*;

/**
 * A simple Web Client.
 **/

public class WClient {
  public static void main(String[] args) {
    try {

      // Check the arguments
      if ((args.length != 1) && (args.length != 2))
        throw new IllegalArgumentException("Wrong number of arguments");
      // Get an output stream to write the URL contents to
      OutputStream wrFile;
      if (args.length > 1) 
        wrFile = new FileOutputStream(args[1]);
      else
        wrFile = System.out;
      
      // Use the URL class to parse the user-specified URL into its 
      // various parts: protocol, host, port, filename. Check the protocol
      URL url = new URL(args[0]);
      String protocol = url.getProtocol();
      if (!protocol.equals("http"))
        throw new IllegalArgumentException("URL must use 'http:' protocol");
      String host = url.getHost();
      int port = url.getPort();
      if (port == -1) port = 80;  // if no port, use the default HTTP port
      String filename = url.getFile();

      // Open a network socket connection to the specified host and port
      Socket socket = new Socket(host, port);

      // Get input and output streams for the socket
      InputStream rdServer = socket.getInputStream();
      PrintWriter wrServer = 
        new PrintWriter(new OutputStreamWriter(socket.getOutputStream()));
      
      // Send the HTTP GET command to the Web server, specifying the file.
      // This uses an old and very simple version of the HTTP protocol
      wrServer.println("GET " + filename + " HTTP/1.0");
      wrServer.println(""); // followed by newline
      wrServer.flush();  // Send it right now!
      
      // Now read the server's response, and write it to the file
      byte[] buffer = new byte[4096];
      int bytesRead;
      int i;

      // read first block with header info (4096 should do)
      bytesRead = rdServer.read(buffer);
      if (bytesRead > 0) {
        for (i=0; i < bytesRead; i++) {
          System.out.write (buffer[i]);
	  if ((i >= 4) && 
              (buffer[i-3] == '\r') && (buffer[i-2] == '\n') &&
              (buffer[i-1] == '\r') && (buffer[i] == '\n')) {
            break; // for loop
          }
        }
        i++;
        // now copy rest of first buffer to file
        wrFile.write(buffer, i, bytesRead - i);
        // now read rest of socket and copy to file
        while((bytesRead = rdServer.read(buffer)) != -1)
          wrFile.write(buffer, 0, bytesRead);
      }
      
      // When the server closes the connection, we close our stuff
      socket.close();
      wrFile.close();
    }
    catch (Exception e) {    // Report any errors that arise
      System.err.println(e);
      System.err.println("Usage: java WClient <URL> [<filename>]");
    }
  }
}

