// WServer - Willem's Java WebServer

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

/**
 * A simple Web Server using threads.
 **/

public class WServer extends Thread {

  static File docRoot;
  static String indexFile = "index.html";
  Socket connection;

  public WServer(Socket s) {
    connection = s;
  }

  public static void main(String args[]) {

    int srvPort;
    ServerSocket srvSocket;

    try {

      // Check the arguments
      if ((args.length != 1) && (args.length != 2))
        throw new IllegalArgumentException("Wrong number of arguments");
      
      // get the Document root
      docRoot = new File(args[0]);

      // Get the port to listen on
      if (args.length == 2) {
        try {
          srvPort = Integer.parseInt(args[1]);
        }
        catch (Exception e) {
          throw new IllegalArgumentException("Non valid portnumber");
        }
        if (srvPort < 0 || srvPort > 65535) {
          throw new IllegalArgumentException("Non valid portnumber");
        }
      } 
      else { 
        srvPort = 80;
      }

      // Create a ServerSocket to listen on that port.
      try {
        srvSocket = new ServerSocket(srvPort);
        System.out.println("");
        System.out.println("WServer started");
        System.out.println("using socketport: " + srvSocket.getLocalPort());
        System.out.println("document root is: " + docRoot);
        System.out.println("");
  
        while (true) {
          WServer ws = new WServer(srvSocket.accept());
          ws.start();
        }
      }
      catch (Exception e) {
        System.err.println("Server couldn't be started properly");
      }
    }

    catch (Exception e) { // Report any errors that arise
      System.err.println(e);
      System.err.println("Usage: java WServer <document root> [<port>]");
    }

  }

  public void run() {
  
    String method;
    String contentType;
    String version = "";
    File docFile;
  
    try {
      // Get input and output streams to talk to the client from the socket
      BufferedReader rdClient = 
        new BufferedReader(new InputStreamReader(connection.getInputStream()));
      OutputStream wrClient = connection.getOutputStream();
      PrintWriter prClient = new PrintWriter(new OutputStreamWriter(wrClient));

      String reqLine = rdClient.readLine();
//    System.out.print(reqLine);

      StringTokenizer st = new StringTokenizer(reqLine);
      method = st.nextToken();
      if ((method.equals("GET")) || (method.equals("HEAD"))) {
        String file = st.nextToken();
        if (file.endsWith("/")) file += indexFile;
        contentType = fileContentType(file);
        if (st.hasMoreTokens()) {
          version = st.nextToken();
        }
        // loop through the rest of the input lines 
        
        String headerLine;
        while ((headerLine = rdClient.readLine()) != null) {
          if (headerLine.trim().equals("")) break;        
        }

        try {
          docFile = new File(docRoot, file.substring(1));

          if (docFile.exists()) {

  	    InputStream rdFile;
            rdFile = new FileInputStream(docFile);

            System.out.println(reqLine + " - 200 OK");
 
            if (version.startsWith("HTTP/")) { // send a MIME header
              Date now = new Date();
              prClient.print("HTTP/1.0 200 OK\r\n");
              prClient.print("Server: WServer 1.0\r\n");
              prClient.print("Date: " + ietfFormat(now) + "\r\n");
	      prClient.print("Last-Modified: " + ietfFormat(docFile.lastModified()) + "\r\n");
              prClient.print("Content-length: " + docFile.length() + "\r\n");
              prClient.print("Content-type: " + contentType + "\r\n");
              prClient.print("\r\n");
  	      prClient.flush();
            }  // end try

            // for HEAD method skip writing data
            if (method.equals("GET")) {
              // Now read the file and write to the connected client
              byte[] buffer = new byte[4096];
              int bytesRead;
              while((bytesRead = rdFile.read(buffer)) != -1)
                wrClient.write(buffer, 0, bytesRead);
	      wrClient.flush();
            }

            // When the file is done, we close the I/O
            prClient.close();
            wrClient.close();
            rdFile.close();

          }
          else {

            System.out.println(reqLine + " - 404 File Not Found");

            if (version.startsWith("HTTP/")) {  // send a MIME header
              Date now = new Date();
              prClient.print("HTTP/1.0 404 File Not Found\r\n");
              prClient.print("Server: WServer 1.0\r\n");
              prClient.print("Date: " + ietfFormat(now) + "\r\n");
              prClient.print("Content-length: 111\r\n");
              prClient.print("Content-type: text/html" + "\r\n");
              prClient.print("\r\n");
            } 
            prClient.println("<HTML><HEAD><TITLE>File Not Found</TITLE></HEAD>");
            prClient.println("<BODY><H1>HTTP Error 404: File Not Found</H1></BODY></HTML>");
            prClient.close();
          }

        }  // end try
        catch (IOException e) {  // probably client closed connection
        }
      } 
      else {  // method does not equal "GET" or "HEAD"

        System.out.println(reqLine + " - 501 Not Implemented");

        if (version.startsWith("HTTP/")) {  // send a MIME header
          Date now = new Date();
          prClient.print("HTTP/1.0 501 Not Implemented\r\n");
          prClient.print("Server: WServer 1.0\r\n");
          prClient.print("Date: " + ietfFormat(now) + "\r\n");
          prClient.print("Content-length: 114\r\n");
          prClient.print("Content-type: text/html" + "\r\n"); 
          prClient.print("\r\n");
        }       
        prClient.println("<HTML><HEAD><TITLE>Not Implemented</TITLE></HEAD>");
        prClient.println("<BODY><H1>HTTP Error 501: Not Implemented</H1></BODY></HTML>");
        prClient.close();
      }
    }
    catch (IOException e) {
    }

    try {
      connection.close();
    }
    catch (IOException e) {
    }

  }
  
  public String fileContentType(String name) {
    if (name.endsWith(".html") || name.endsWith(".htm")) 
      return "text/html";
    else if (name.endsWith(".gif")) 
      return "image/gif";
    else if (name.endsWith(".jpg") || name.endsWith(".jpeg")) 
      return "image/jpeg";
    else if (name.endsWith(".png")) 
      return "image/png";
    else if (name.endsWith(".class")) 
      return "application/octet-stream";
    else if (name.endsWith(".txt")) 
      return "text/plain";
    else if (name.endsWith(".log")) 
      return "text/plain";
    else if (name.endsWith(".java")) 
      return "text/plain";
    else if (name.endsWith(".c")) 
      return "text/plain";
    else if (name.endsWith(".h")) 
      return "text/plain";
    else 
      return "text/plain";
  }

  public String ietfFormat(Date date) {

    DateFormat formatter = new SimpleDateFormat ("EEE, dd MMM yyyy HH:mm:ss", Locale.getDefault());
    TimeZone tz = TimeZone.getTimeZone("GMT");
    formatter.setTimeZone(tz);
    String dateStr = formatter.format(date);

    return (dateStr + " GMT");
  }

  public String ietfFormat(long date) {

    Date dateObj = new Date(date);
    return (ietfFormat(dateObj));
  }

}

